# Search for gravitational-lensing signatures in the full third observing run of the LIGO–Virgo network
## Python script for GOLUM type II image search follow-up Figure 9

##############################################################################################################################################
#"Search for gravitational-lensing signatures in the full third observing run of the LIGO–Virgo network" is the second paper from the LIGO,  #
# Virgo, and KAGRA (LVK) Collaborations searching for possible lensing features from gravitational waves during the full third observing run #
# (O3a + O3b) which lasted from 1st April # 2019 to 27th March 2020.                                                                         #
##############################################################################################################################################

###########################################
# What is lensing of gravitational waves? #
###########################################

# General relativity tells us that gravity curves space and time, affecting everything that carries mass or energy, and gravitational waves 
# are no exception. Therefore, we expect that as they travel cosmological distances, gravitational waves will inevitably encounter massive 
# astrophysical objects that would act as lenses, bending their trajectories and even distorting the shapes of their signals. In the paper, 
# we have implemented multiple analyses to search for the still undetected lensing signatures in the gravitational-wave signals detected in 
# O3a+b. 

# This python script is part of the data release for the paper. It focuses on the GOLUM type II image search analysis, and shows you how to reproduce 
# the results shown in Figure 3 of the paper.

########################################
# GOLUM Type II image search follow up #
########################################

#######################################################################
# In this analysis, we analyze the data using a template allowing for #
# an extra global phase shift parametrized by the Morse factor. This  #
# is a typical lensing signature and can be found in some cases.      #
# Two events (GW190412 and GW200129-065458) seem to have some         #
# support. Further investigation show that only GW190412 can possibly #
# lead to detectable type II image features. To make sure what we see #
# is not due to some waveform systematic, we inject the signals with  #
# different waveforms and see if our results are stable for different #
# waveforms.                                                          #
#######################################################################

import json 
import matplotlib.pyplot as plt

########################
#                      #
# SET PLOTTING OPTIONS #
#                      #
########################

import matplotlib as mpl
from numpy import sqrt


# DETERMINE FIGURE SIZE
ratio = 0.0
golden_mean = (sqrt(5)-1.0)/2.0

fig_width_pt = 2.*242.26653 # Get this from LaTeX using \showthe\columnwidth
inches_per_pt = 1.0/72.27 # Convert pt to inch
fig_ratio = ratio if ratio != 0.0 else golden_mean # Aesthetic ratio, unless specified through ratio
fig_width = fig_width_pt*inches_per_pt # width in inches
fig_height = fig_width*golden_mean # height in inches
fig_size =  [fig_width,fig_height]

# COLOR SCHEME

# LOAD PLOTTING OPTIONS
params = {
        'text.usetex': True,
        'font.family': 'cm',
        'font.size': 16,
        'axes.labelsize': 16,
        'legend.fontsize': 16,
        'xtick.labelsize': 16,
        'ytick.labelsize': 16,
        'axes.grid' : True,
        'lines.markersize' : 14,
        'lines.linewidth' : 2,
        'axes.formatter.useoffset': False,
        'figure.figsize': fig_size
        }

mpl.rcParams.update(params)

######################
# core of the script #
######################

# read in the info
with open('o3b_lensing_datafile_figure9_analysis_different_waveforms_GW190412.json', 'r') as f:
    data = json.load(f)

waveforms = ['IMRPhenomPv3HM', 'SEOBNRv4PHM']
types= ['I', 'II']

plt.figure()
for wf in waveforms:
    for typ in types:
        if typ in ['I']:
            plt.hist(data['%s_type_%s'%(wf, typ)], bins = 50, density = True,
            histtype = 'step', label = '%s, Type %s'%(wf, typ),
            linestyle = 'dashed')
        else:
            plt.hist(data['%s_type_%s'%(wf, typ)], bins = 50, density = True,
            histtype = 'step', label = '%s, Type %s'%(wf, typ),
            linestyle = 'dotted')
plt.hist(data['real_event'], bins = 50, density = True,
         histtype = 'step', label = 'Real event')  
plt.legend(loc = 'upper left', prop = {'size' : 10})
plt.grid()
plt.xlabel(r'$n_1$')
plt.ylabel(r'P($n_1$)')
plt.savefig('o3b_lensing_dataproduct_figure_9_type_II_image_follow_u_GW190412_different_waveforms.pdf', bbox_inches = 'tight')
plt.show()